# This file was auto-generated by Fern from our API Definition.

import typing

GetLiveHistoricalOrReverseWhoisRequestWhois = typing.Union[typing.Literal["live", "historical", "reverse"], typing.Any]
