# This file was auto-generated by Fern from our API Definition.

import typing

GetDnsRecordsRequestType = typing.Union[
    typing.Literal["A", "AAAA", "MX", "NS", "CNAME", "SOA", "SPF", "TXT", "ALL"], typing.Any
]
